/ ########## TimeStored Training Course  #######
/ http://www.timestored.com/kdb-training
/ trades.q - http://code.kx.com/wsvn/code/contrib/cburke/start/trades.q

/ create an empty table with types defined to enforce schema.
trade:([]  date:`date$();  
        time:`time$();  
        sym:`symbol$();  
        price:`float$();  
        size:`int$();  
        cond:`char$());

/ we are going to create some semi-sensible random data so
/ initialise a few parameters that set what data is generated.
syms:`IBM`MSFT`UPS`BAC`AAPL`GOOG`KX`RBS`MS`GOOG`AA`C`A`B`D`E`F;
day:5;                / number of days
cnt:count syms;      / number of syms
l:10000*cnt*day;      / total number of trade, 100000 ticks per day

/ generate times between 9-30 and 5.30
time:"t"$raze (cnt*day)#enlist asc 09:30:00+10000?08:00t;
time+:l?1000;

/ create rectangular matrix of values and insert into the trade table
/ notice the order of the values match the column order.
`trade insert (.z.d-l?day; time; l?syms; 50.0f+l?60f; 100*l?1000; l?" ABC");


trade:`date`time xasc trade;  / sort on time within date


/ create quote table that kind of matches trade
/ we want twice as many quotes as trades and for bid/ask to be cose to trade prices.
n:l*2;
quote:delete price from update date, time, sym, bid:price*0.99, ask:price*1.01 from n?trade;
quote:`date`time xasc update asize:n?1000, bsize:n?1000 from quote;

/ For every sym in the trade table, lets have a stock table that
/ contains the sector and number of employees that firm has.
b:exec distinct sym from trade;
c: count b;
stock:([sym:b] sector:c?`Financial`Tech`Financial; 	employees:100+c?1000);


/ ### Stored Procedures

/ as seen in qsql all selct statements consist of 4 parts
/ ?[t; c; b; a]
/ select a1, a2, by b from t where c 


/ ### Functions 

/ merely filter on the where clause and return all rows that match.
getTradesFor:{ [s] select from trade where sym in s};

/ filter to a group of syms
/ group the price column by date sym
/ apply the aggregates
getOhlc:{ [s] select open:first price, high:max price, low:min price, 
    close:last price, volume:sum `long$size by date,sym from trade where sym in s};
    

/ getOhlc `RBS;

    
/ filter using conditions for date and sym
/ group by sym and a time component
/ calculate the average price within each time window
/ and the volume weighted average price within each time window
/ notice how easy kdb makes these queries.    
getHourlyPrice:{ [s; dts]
    select avgPrice:avg price, vwap:size wavg price 
        by sym, time.hh
        from trade where sym in s,date within dts };


/ getHourlyPrice[`AAPL; .z.d + 0 1]

/ Get the maximum, minimum, average price stats ever 
getPriceStats:{ [s; dts]
    select maxPrice:max price, minPrice:min price, 
    avgPrice:avg price, volume:sum `long$size
    by sym from trade where sym in s };


/ getPriceStats[`AAPL; .z.d + 0 1]        
    
    
    

/ filter to a group of syms
/ group the price column by date sym
/ apply the aggregates
getOhlcWithin:{ [s; dts] 
    a:select open:first price, high:max price, low:min price, 
        close:last price, volume:sum `long$size by date,sym 
        from trade where sym in s, date within dts;
    allDates: dts[0] + til dts[1]-dts[0];
    allKeys: flip `date`sym!flip allDates cross (),s; 
    allKeys lj a};
/ fills getOhlcWithin[`AAPL`GOOG; .z.d + 0 7]    
    
    
    
    
    

    